#!/usr/bin/perl

use warnings;
use strict;

use Digest::SHA1  qw(sha1_hex);

# Download CryptoPhoto Perl lib from https://cryptophoto.com/admin/api
use CryptoPhotoUtils;
use CGI;

my $server      = "https://cryptophoto.com";
my $private_key = "xxx"; # Your CryptoPhoto Private API Key
my $public_key  = "yyy"; # Your CryptoPhoto Public API Key
my $salt        = "random-salt-string";
my $uid         = "userId"; # Your users internal ID

my $cgi = CGI->new();
my $_POST = $cgi->Vars();

my $step = "login";
my $error = '';
my $sid = '';                     

if (exists $_POST->{'username'}) {
  # check user name
  if ($_POST->{'username'} eq 'demo') {
    # all OK
    # do CryptoPhoto authentication
    $step = 'cp_authentication';
  } else {
    $error = "Login credentials invalid";
  }
}
elsif (exists $_POST->{'cryptophoto'}) {
  $step = 'cp_authentication';

  # check for POSTed CP Token codes
  if (
    $_POST->{'token_selector'} &&
    (($_POST->{'token_response_field_row'} && $_POST->{'token_response_field_col'}) || $_POST->{'cp_phc'})) {

    my $cp = new CryptoPhotoUtils($server, $private_key, $public_key, sha1_hex($uid.$salt));

    my $verify = $cp->verify_response($_POST->{'token_selector'}, $_POST->{'token_response_field_row'}, 
                                      $_POST->{'token_response_field_col'}, $_POST->{'cp_phc'}, $ENV{'REMOTE_ADDR'});

    if ($verify->{'is_valid'}) {# CP Token codes verified
      $step = 'success';
    }
    else {
      $error = 'Codes invalid.';
    }
  } else {
    $error = 'Missing required codes. Invalid authentication';
  }
}

if ($step eq 'cp_authentication') {

  my $cp = new CryptoPhotoUtils($server, $private_key, $public_key, sha1_hex($uid.$salt));

  my $session = $cp->start_session($ENV{'REMOTE_ADDR'}, 1);

  if ($session->{'is_valid'}) { # API call was successful
    if ($session->{'has_token'}) {
      $sid = $session->{'sid'};
    } else {
      $step = 'success';
    }
  } else {
    $error = $session->{'error'} . ( $session->{'errdesc'} ? ' ' . $session->{'errdesc'} : '' );
  }
}
  
my $display = '';

if ($error) {
  $display .= '<p>Error: ' . $error . '</p>';
}
warn "Step = $step \n";
if ($step eq 'login') {
  $display .= 'Username: <input type="text" name="username" value="demo"><br>' .
    '<input type="submit" value="Login">';
} elsif ($step eq 'cp_authentication') {
  $display .= '<input type="hidden" name="cryptophoto">' .
    '<div id="cp_widget" style="margin: 10px auto;width:300px">Loading...</div>' .
    '<input type="submit" value="Continue">' .
    '<script type="text/javascript"' .
    'src="'. $server .'/api/challenge?sd='.$sid.'"></script>';
  
} elsif ($step eq 'success') {
  $display .= '<p>You have successfully login.</p>';
}
 
my $out=<<HTML;
Content-Type: text/html; charset=utf-8

<!DOCTYPE html>
<html>
<head>
<meta charset='UTF-8'>
<title>User Login</title>
</head>

<body style='text-align:center'>
  <h2>Login</h2>
  <form action='' method='post'>
       $display
  </form>
</body>
</html>
HTML

print $out;